package de.hft.stuttgart.citydoctor2.healing;

import java.io.File;
import java.io.IOException;

import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;

import de.hft.stuttgart.citydoctor2.check.CheckId;
import de.hft.stuttgart.citydoctor2.check.Checker;
import de.hft.stuttgart.citydoctor2.check.Requirement;
import de.hft.stuttgart.citydoctor2.check.RequirementConfiguration;
import de.hft.stuttgart.citydoctor2.check.ValidationConfiguration;
import de.hft.stuttgart.citydoctor2.datastructure.Building;
import de.hft.stuttgart.citydoctor2.datastructure.CityDoctorModel;
import de.hft.stuttgart.citydoctor2.exceptions.CityDoctorWriteException;
import de.hft.stuttgart.citydoctor2.healer.Healer;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParseException;
import de.hft.stuttgart.citydoctor2.parser.InvalidGmlFileException;

public class HealPolygonWithoutSurfaceTest {
	
	@Rule
	public TemporaryFolder folder = new TemporaryFolder();

	@Test
	public void testHealPolygonWithoutSurface() throws CityGmlParseException, IOException, InvalidGmlFileException, CityDoctorWriteException {
		ValidationConfiguration config = ValidationConfiguration.loadStandardValidationConfig();
		config.getRequirements().put(Requirement.R_SE_POLYGON_WITHOUT_SURFACE.getId(), new RequirementConfiguration(true));
		String path = "src/test/resources/MissingGroundSurface.gml";
		CityDoctorModel model = TestUtil.loadCityModel(path, config);

		Checker c = new Checker(config, model);
		c.runChecks();
		
		Building building = model.getBuildings().get(0);
		Assert.assertTrue(building.containsError(CheckId.C_SE_POLYGON_WITHOUT_SURFACE));

		Healer healer = new Healer(c);
		healer.healCityObject(model.getBuildings().get(0));
		
		Assert.assertFalse(building.containsError(CheckId.C_SE_POLYGON_WITHOUT_SURFACE));
		
		File outputFile = folder.newFile();
		model.saveAs(outputFile.toString(), false);
		
		CityDoctorModel loadedModel = TestUtil.loadCityModel(outputFile.toString(), config);
		c = new Checker(config, loadedModel);
		c.runChecks();
		building = loadedModel.getBuildings().get(0);
		Assert.assertFalse(building.containsError(CheckId.C_SE_POLYGON_WITHOUT_SURFACE));
	}

}
