package de.hft.stuttgart.citydoctor2.healer.genetic.gui;

import java.util.List;

import de.hft.stuttgart.citydoctor2.check.Checker;
import de.hft.stuttgart.citydoctor2.datastructure.BoundingBox;
import de.hft.stuttgart.citydoctor2.datastructure.Geometry;
import de.hft.stuttgart.citydoctor2.healer.Healer;
import de.hft.stuttgart.citydoctor2.healer.HealingPlan;
import de.hft.stuttgart.citydoctor2.healer.genetic.FitnessEvaluation;
import de.hft.stuttgart.citydoctor2.math.Vector3d;
import de.hft.stuttgart.citydoctor2.utils.Copy;
import javafx.scene.image.Image;

public class VisualFitness implements FitnessEvaluation<HealingPlan> {
	
	private int xRotSteps = 6;
	private int yRotSteps = 12;
	
	private GeneticsGUI gui;
	private Geometry geom;
	private List<Image> originalImages;
	private Checker c;
	private double distance;
	
	public VisualFitness(GeneticsGUI gui, Geometry geom, Checker c) {
		this.gui = gui;
		this.geom = geom;
		this.c = c;
		
		BoundingBox b = geom.calculateBoundingBox();
		double cameraDistance = 2.0d;
		Vector3d objectSizes = b.getDiagonal();
		double objectSize = max(objectSizes.getX(), objectSizes.getY(), objectSizes.getZ());
		double cameraView = 2.0d * Math.tan(0.5d * Math.toRadians(30)); // Visible height 1 meter in front
		distance = cameraDistance * objectSize / cameraView;
		distance += 0.5d * objectSize; // Estimated offset from the center to the outside of the object
		distance = -distance;
		
		originalImages = gui.createScreenshots(geom, distance, xRotSteps, yRotSteps);
	}
	
	private double max(double a, double b, double c) {
		double temp = Math.max(a, b);
		return Math.max(temp, c);
	}


	@Override
	public double evaluate(HealingPlan plan) {
		Healer healer = new Healer(c, plan);
		Geometry copy = Copy.copy(geom);
		healer.heal(copy, copy.getParent());
		if (copy.containsAnyError()) {
			System.out.println(plan);
			System.out.println("Fitness: 0");
			return 0;
		}
		List<Image> compareImages = gui.createScreenshots(copy, distance, xRotSteps, yRotSteps);
		if (originalImages.size() != compareImages.size()) {
			throw new IllegalStateException("Number of images differs");
		}
		double value = 0;
		for (int i = 0; i < originalImages.size(); i++) {
			Image orig = originalImages.get(i);
			Image compare = compareImages.get(i);
			value += ImageComparison.compare(orig, compare);
		}
		double fitness = value / (xRotSteps * yRotSteps);
		System.out.println("Fitness: " + fitness);
		return fitness;
	}

}
